#include "config.h"
#include "autoconf.h"
#include "macros.h"
#include "network.h"
#include "ag.h"
#include "readconf.h"
#include "share.h"
#include <signal.h>

int debug = 0;
int background = 0;
int running = 1;
int status = AG_STAT_DISCONNECTED;
struct timeval uptime;
FILE *logfile = NULL, *_logfile = NULL;
#ifdef USE_CONTEXTS
int _ncontext=0;
unsigned char _context[_NCX][128];
#endif


/* handles SIGTERM, SIGINT signals */
void sighandler()
{
#ifdef USE_CONTEXTS
	int i=0;
#endif

	if(running)
	{
		NOTICE("Exiting... please wait.");
#ifdef USE_CONTEXTS
		if(debug)
		{
			NOTICE("Context backlog:");
			for(;i<_NCX;i++)
				NOTICE("-> %s", _context[(i+_ncontext)%_NCX]);
		}
#endif
		if(status != AG_STAT_DISCONNECTED)
		{
			NOTICE("Disconnecting...");
		}
		running = 0;
	}
	else
	{
		/* second signal - force exit */
		NOTICE("Dirty exit.");
		exit(0);
	}
}


/* handles crashes ;) */
void sigcrash()
{
#ifdef USE_CONTEXTS
	static int i=0;
#endif

	printf("\n\nCrashed.\n");
#ifdef USE_CONTEXTS
	printf("context backlog:\n");
	for(;i<_NCX;i++)
		printf("-> %s\n", _context[(i+_ncontext)%_NCX]);
#endif
	exit(666);
}


int main(int argc, char **argv)
{
	struct timeval tv;
	int i,t[3];

#ifdef USE_CONTEXTS
	/* null-terminate contexts */
	for(i=0;i<_NCX;i++)
		_context[i][0] = 0;
#endif
	context;

	/* simple arg parser */
	for(i=1;i<argc;i++)
	{
		if(!strcmp(argv[i], "--debug") || !strcmp(argv[i], "-d"))
		{
			debug = 1;
		}
		else if(!strcmp(argv[i], "--background") || !strcmp(argv[i], "-b"))
		{
			background = 1;
		}
		else if(!strcmp(argv[i], "--version") || !strcmp(argv[i], "-v"))
		{
			printf("fags version " VERSION "\n");
			return 0;
		}
		else if(!strcmp(argv[i], "--help") || !strcmp(argv[i], "-h"))
		{
			printf(
				"Usage: fags [OPTION]\n"
				"\n"
				"  -d, --debug        debug mode\n"
				"  -b, --background   run in background\n"
				"  -h, --help         display this help and exit\n"
				"  -v, --version      output version information and exit\n"
				"\n"
				"Report bugs to <dinkles@tty0.org>\n"
				);
			return 0;
		}
		else
		{
			NOTICE("Unknown argument '%s'", argv[i]);
			return 1;
		}
	}

	/* allow root if debugging */
	if(debug == 0 && getuid() == 0) {
		NOTICE("FAGS can't be run as root.");
		return 666;
	}

	switch(config_setup_homedir())
	{
	case 1:
		printf(	"\n"
				"FAGS has installed a template config file in ~/.fags/\n"
				"Please edit this file and start fags again.\n");
		return 1;
	case 2:
		return 2;
		break;
	}

	if(background)
	{
		switch(fork())
		{
		case -1:
			NOTICE("Unable to fork!");
			return 0;
		case 0:
			NOTICE("Forked.");
			break;
		default:
			exit(0);
		}
	}

	/* set signal handlers ... */
	signal(SIGTERM, &sighandler);
	signal(SIGINT, &sighandler);
	signal(SIGABRT, &sigcrash);
	signal(SIGQUIT, &sigcrash);
	signal(SIGSEGV, &sigcrash);
#ifdef SIGBUS
	signal(SIGBUS, &sigcrash);
#endif
	signal(SIGPIPE, SIG_IGN);
	signal(SIGCHLD, SIG_IGN);

	gettimeofday(&uptime, NULL);

	if(!background)
		printf(
			"Welcome to FAGS version " VERSION ", Copyright (c) 2002 dinkles <dinkles@tty0.org>\n"
			"FAGS comes with ABSOLUTELY NO WARRANTY; for details read the LICENSE file\n"
			"This is free software, and you are welcome to redistribute it\n"
			"under certain conditions; read the LICENSE file for details.\n\n"

			"No reverse engineering or decompilation of any Audiogalaxy, Inc.\n"
			"code took place to create this program.\n\n"

#ifdef USE_VORBIS
			"Compiled with Ogg Vorbis support.\n"
#endif
#ifndef USE_CONTEXTS
			"Compiled without context support.\n"
#endif
			);

	if(!config_read())
	{
		NOTICE("Config file error!");
		return 2;
	}

	if(conf.logfile[0])
	{
		_logfile = fopen(conf.logfile, "a");
		if(!_logfile)
			NOTICE("Couldn't open '%s': %s", conf.logfile, ERRMSG());
		else
			NOTICE("Opened logfile '%s'.", conf.logfile);

		LOG_ENABLE();
		if(logfile)
			fprintf(logfile, "-- FAGS started.\n");
		}

	if(!conf.hostname[0])
	{
		if(gethostname(conf.hostname, sizeof(conf.hostname))) {
			NOTICE("Couldn't detect your hostname, please edit ~/.fags/config!");
			return 3;
		}
		NOTICE("Detected hostname: %s", conf.hostname);
	}

	if(!conf.ip[0])
	{
		struct in_addr *tmp = resolve_host(conf.hostname);

		if(!tmp) {
			NOTICE("Couldn't resolve your hostname, please edit ~/.fags/config!");
			return 4;
		}
		ag_strncpy(conf.ip, inet_ntoa(*tmp), sizeof(conf.ip));
		NOTICE("Detected IP Address: %s", conf.ip);
	}

	if(conf.share_downloads && !path_is_shared(conf.download_dir))
	{
		NOTICE("The download directory must be shared! Disabling sharing of new downloads...");
		conf.share_downloads = 0;
	}

	context;
	share_load_db();
	share_list(conf.sharelist);
	share_status();
	share_save_db();

	ag_main_loop();

	share_save_db();
	share_free();
	event_unregister_all();
	config_free();

	gettimeofday(&tv, NULL);
	tv.tv_sec -= uptime.tv_sec;
	t[0] = tv.tv_sec/3600;
	t[1] = (tv.tv_sec/60)%60;
	t[2] = tv.tv_sec%60;
	NOTICE("Uptime: %d hour%s, %d minute%s, and %d second%s.",
			t[0], t[0]==1?"":"s",
			t[1], t[1]==1?"":"s",
			t[2], t[2]==1?"":"s");
	NOTICE("Clean exit.");

	if(logfile)
		fprintf(logfile, "-- FAGS ended.\n");

	if(_logfile)
		fclose(_logfile);

#ifdef USE_MALLOC_DEBUG
	malloc_dump();
	malloc_free_all();
#endif

	return 0;
}

