#include "config.h"
#include "macros.h"
#include "readconf.h"
#include <sys/stat.h>
#include <sys/types.h>
#include <fcntl.h>
#ifdef HAVE_REALPATH
#	include <sys/param.h>
#	ifndef MAXPATHLEN
#		define MAXPATHLEN 1024
#	endif
#endif

struct confstruct conf;


void config_free()
{
	struct confshare *s = conf.sharelist;
	struct confshare *next;

	context;

	while(s)
	{
		next = s->next;
		ag_free(s);
		s = next;
	}
}


int opt_is_true(unsigned char *str)
{
	str[4] = 0;
	if(!strcasecmp(str, "true"))
		return 1;
	str[3] = 0;
	if(!strcasecmp(str, "yes"))
		return 1;
	if(atoi(str))
		return 1;
	return 0;
}


/* expands ~ into /home/whatever/ */
unsigned char *expand(unsigned char *path, int do_realpath, int is_dir)
{
	static unsigned char npath[MAX_PATH];
	char *home;
	int len;
#ifdef HAVE_REALPATH
	char resolved_path[MAXPATHLEN];
#endif

	if(*path == '~')
	{
		home = getenv("HOME");
		len = snprintf(npath, MAX_PATH, "%s/%s", home?home:".", path+1);
	}
	else
	{
		len = snprintf(npath, MAX_PATH, "%s", path);
	}
	if(len == -1)
	{
		errno = ENAMETOOLONG;
		return NULL;
	}

#ifdef HAVE_REALPATH
	if(do_realpath)
	{
		if(realpath(npath, resolved_path))
			ag_strncpy(npath, resolved_path, MAX_PATH);
		else
			return NULL;
	}
#endif

	len = strlen(npath);

	if(is_dir && len && npath[len-1] != '/' && len < (MAX_PATH-1))
	{
		npath[len] = '/';
		npath[len+1] = 0;
	}

	errno = 0;

	return npath;
}


int config_read()
{
	FILE *fd;
	unsigned char str[256], cmd[32], opt[224], act[224], *ptr;
	int line=0,i;
	struct confshare *s=NULL;

	memset(&conf, 0, sizeof(struct confstruct));

	conf.statusfile_interval = 15000;
	conf.share_downloads = 1;

	fd = fopen(etc_file("config"), "r");
	if(!fd) {
		NOTICE("Unable to open '%s': %s", etc_file("config"), ERRMSG());
		return 0;
	}

	while(fgets(str, sizeof(str), fd)) {
		line++;
		if(str[0] == 0 || str[0] == '#' || str[0] == ';')
			continue;

		i = strlen(str);
		if(i == 1) continue;			/* skip empty lines */
		if(i > 0 && str[i-1] == '\n')	/* strip trailing newline unsigned char */
			str[i-1] = 0;

		memset(&opt, 0, sizeof(opt));
		if(sscanf(str, "%32s %223c", cmd, opt) != 2) {
			NOTICE("Parse error on line %d in config", line);
			return 0;
		}

		/* user */
		if(!strcasecmp(cmd, "user")) {
			ag_strncpy(conf.username, opt, sizeof(conf.username));

		/* pass */
		} else if(!strcasecmp(cmd, "pass")) {
			ag_strncpy(conf.password, opt, sizeof(conf.password));

		/* share*/
		} else if(!strcasecmp(cmd, "share")) {
			if(s)
			{
				s->next = ag_malloc(sizeof(struct confshare));
				s = s->next;
			}
			else
			{
				s = conf.sharelist = ag_malloc(sizeof(struct confshare));
			}
			s->next = NULL;

			ptr = expand(opt, 1, 1);
			if(!ptr)
				NOTICE("Can't add share directory '%s': %s", opt, ERRMSG());
			else
				ag_strncpy(s->path, ptr, MAX_PATH);

		/* download_dir */
		} else if(!strcasecmp(cmd, "download_dir")) {
			ptr = expand(opt, 1, 1);
			if(!ptr)
				NOTICE("Can't set download directory '%s': %s", opt, ERRMSG());
			else
				ag_strncpy(conf.download_dir, ptr, MAX_PATH);

		/* event */
		} else if(!strcasecmp(cmd, "event")) {
			memset(&act, 0, sizeof(act));
			if(sscanf(opt, "%32s %223c", cmd, act) != 2) {
				NOTICE("Parse error on event cmd, line %d in config", line);
				return 0;
			}
			ptr = expand(act, 1, 0);
			if(!ptr)
				NOTICE("Can't register event action '%s': %s", act, ERRMSG());
			else
				event_register(cmd, ptr);

		/* hostname */
		} else if(!strcasecmp(cmd, "hostname")) {
			ag_strncpy(conf.hostname, opt, sizeof(conf.hostname));

		/* ip */
		} else if(!strcasecmp(cmd, "ip")) {
			ag_strncpy(conf.ip, opt, sizeof(conf.ip));

		/* share_downloads*/
		} else if(!strcasecmp(cmd, "share_downloads")) {
			conf.share_downloads = opt_is_true(opt);

		/* capitalize_files */
		} else if(!strcasecmp(cmd, "capitalize_files")) {
			conf.capitalize_files = opt_is_true(opt);

		/* logfile */
		} else if(!strcasecmp(cmd, "logfile")) {
			ptr = expand(opt, 0 ,0);
			if(!ptr)
				NOTICE("Can't set log file '%s': %s", opt, ERRMSG());
			else
				ag_strncpy(conf.logfile, ptr, MAX_PATH);

		/* bandwidth_download */
		} else if(!strcasecmp(cmd, "bandwidth_download")) {
			conf.bandwidth_download = atoi(opt);

			if(conf.bandwidth_download != 0 && conf.bandwidth_download < 1024)
				conf.bandwidth_download = 1024;

		/* bandwidth_upload */
		} else if(!strcasecmp(cmd, "bandwidth_upload")) {
			conf.bandwidth_upload = atoi(opt);

			if(conf.bandwidth_upload != 0 && conf.bandwidth_upload < 1024)
				conf.bandwidth_upload = 1024;

		/* status file */
		} else if(!strcasecmp(cmd, "statusfile")) {
			ptr = expand(opt, 0 ,0);
			if(!ptr)
				NOTICE("Can't set status file '%s': %s", opt, ERRMSG());
			else
				ag_strncpy(conf.statusfile, ptr, MAX_PATH);

		/* statusfile update inteveral */
		} else if(!strcasecmp(cmd, "statusfile_interval")) {
			conf.statusfile_interval = atoi(opt);

			if(conf.statusfile_interval < 1)
				conf.statusfile_interval = 1;

			conf.statusfile_interval *= 1000;

		} else {
			NOTICE("Unknown option on line %d in fags.conf", line);
		}
	}

	fclose(fd);


	if(!conf.username[0] || !conf.password[0])
	{
		NOTICE("No username or password set!");
		return 0;
	}

	if(!conf.sharelist)
	{
		NOTICE("No share directory set!");
		return 0;
	}

	if(!conf.download_dir[0])
	{
		/* default to "." */
		conf.download_dir[0] = '.';
		conf.download_dir[1] = 0;
	}

	/* anyone want me to uncomment this code? :) */
	/*if(conf.bandwidth_upload != 0 && conf.bandwidth_upload < 8192)
	{
		if(conf.bandwidth_download == 0 || conf.bandwidth_download > conf.bandwidth_upload*10)
		{
			conf.bandwidth_download = conf.bandwidth_upload*10;
			NOTICE("Leech protection in effect. Download bandwidth limit set to %d bytes/sec.",
					conf.bandwidth_download);
			NOTICE("Set your upload bandwidth limit to at least 8192 bytes/sec.");
		}
	}*/

	return 1;
}


int config_setup_write(unsigned char *file, int exec, unsigned char *cont)
{
	FILE *fd;
	int len;

	if(!file_exists(file))
	{
		fd = fopen(file, "w");
		if(!fd)
		{
			printf("Can't open '%s': %s\n", file, ERRMSG());
			return 1;
		}

		len = strlen(cont);
		if(fwrite(cont, 1, len, fd) != len)
		{
			printf("Error writing to '%s': %s\n", file, ERRMSG());
			fclose(fd);
			return 1;
		}
		fclose(fd);
		if(exec)
			chmod(file, S_IRUSR|S_IWUSR|S_IXUSR);
		else
			chmod(file, S_IRUSR|S_IWUSR);
		printf("Wrote %s...\n", file);
		return 2;
	}

	return 0;
}


int config_setup_homedir()
{
	int err,ret=0;

	/* create ~/.fags/ */
	if(mkdir(etc_file(""), S_IRUSR|S_IWUSR|S_IXUSR))
	{
		if(errno != EEXIST)
		{
			printf("Can't create directory '%s': %s\n", etc_file(""), ERRMSG());
			return 0;
		}
	} else
		printf("Created directory %s...\n", etc_file(""));

	/* create ~/.fags/scripts/ */
	if(mkdir(etc_file("scripts"), S_IRUSR|S_IWUSR|S_IXUSR))
	{
		if(errno != EEXIST)
		{
			printf("Can't create directory '%s': %s\n", etc_file(""), ERRMSG());
			return 0;
		}
	} else 
		printf("Created directory %s...\n", etc_file("scripts"));


	err = config_setup_write(etc_file("config"), 0,
		"# Login details\n"
		"#\n"
		"user username\n"
		"pass password\n"
		"\n"

		"# Shared directories (at least 1 is required)\n"
		"share /pub/music\n"
		"#share	/pub/music2\n"
		"#share	/mnt/mp3\n"
		"\n"

		"# Where to put downloads (default is \".\")\n"
		"#download_dir /pub/music/download\n"
		"\n"

		"# Global bandwidth limiting in bytes/second\n"
		"# Set to 0 to disable - minimum value is 1024\n"
		"#\n"
		"#bandwidth_download 0\n"
		"#bandwidth_upload   40960\n"
		"\n"

		"# Event scripts\n"
		"#\n"
		"#event connect        ~/.fags/scripts/connect.sh\n"
		"#event disconnect     ~/.fags/scripts/disconnect.sh\n"
		"#event login          ~/.fags/scripts/login.sh\n"
		"#event login_error    ~/.fags/scripts/login_error.sh\n"
		"#event upload_start   ~/.fags/scripts/upload_start.sh\n"
		"#event upload_end     ~/.fags/scripts/upload_end.sh\n"
		"#event download_start ~/.fags/scripts/download_start.sh\n"
		"#event download_end   ~/.fags/scripts/download_end.sh\n"
		"#event delete         ~/.fags/scripts/delete.sh\n"
		"#event play_file      ~/.fags/scripts/playfile.sh\n"
		"#event play_file      ~/.fags/scripts/playfile2.sh\n"
		"#event play_file      ~/.fags/scripts/playfile3.sh\n"
		"\n"

		"# In case hostname and IP detection fails, fill these in\n"
		"#\n"
		"#hostname bogbrush\n"
		"#ip       1.2.3.4\n"
		"\n"

		"# Share downloaded files? (default is yes)\n"
		"#\n"
		"# download_dir must be located under one of\n"
		"# your shared directories for this to work\n"
		"#\n"
		"#share_downloads no\n"
		"\n"

		"# Capitalize filenames? (default is no)\n"
		"#\n"
		"# eg. rename 'inFecTed MUSHRoom - funkaMillion.MP3'\n"
		"# to 'Infected Mushroom - Funkamillion.mp3\n"
		"#\n"
		"#capitalize_files yes\n"
		"\n"

		"# Logfile (comment out if you don't want a logfile)\n"
		"#\n"
		"#logfile ~/.fags/log\n"
		"\n"

		"# Statusfile (comment out if you don't want a statusfile)\n"
		"#\n"
		"#statusfile ~/.fags/status\n"
		"\n"

		"# Statusfile update interval in seconds\n"
		"#\n"
		"#statusfile_interval 15\n"
		"\n"
		);
	if(err == 2)
		ret = 1;

	err = config_setup_write(etc_file("scripts/connect.sh"), 1,
		"#!/bin/sh\n"
		"\n"
		"# connect.sh - args: <server-ip> <server-port>\n"
		"logger -t fags \"Connected to server $1:$2.\"\n"
	);
	if(err == 1)
		ret = 2;

	err = config_setup_write(etc_file("scripts/disconnect.sh"), 1,
		"#!/bin/sh\n"
		"\n"
		"# disconnect.sh - args: none\n"
		"logger -t fags \"Disconnected from server.\"\n"
	);


	if(err == 1)
		ret = 2;
	err = config_setup_write(etc_file("scripts/login.sh"), 1,
		"#!/bin/sh\n"
		"\n"
		"# login.sh - args: <login_string>\n"
		"#konqueror \"http://www.audiogalaxy.com/user/home.php?$1\" &\n"
		"logger -t fags \"Login URL: http://www.audiogalaxy.com/user/home.php?$1\""
	);
	if(err == 1)
		ret = 2;


	if(err == 1)
		ret = 2;
	err = config_setup_write(etc_file("scripts/login_error.sh"), 1,
		"#!/bin/sh\n"
		"\n"
		"# login_error.sh - args: <message>\n"
		"logger -t fags \"Login error: $1.\"\n"
	);
	if(err == 1)
		ret = 2;


	err = config_setup_write(etc_file("scripts/download_start.sh"), 1,
		"#!/bin/sh\n"
		"\n"
		"# download_start.sh - args: <path> <file> <ip>\n"
		"logger -t fags \"Downloading '$2' from $3.\"\n"
	);
	if(err == 1)
		ret = 2;

	err = config_setup_write(etc_file("scripts/download_end.sh"), 1,
		"#!/bin/sh\n"
		"\n"
		"# download_end.sh - args: <path> <file> <avg speed> [song_id]\n"
		"# song_id is only set if share_downloads is enabled\n"
		"logger -t fags \"Download of '$2' completed. Average speed was $3KB/s. (song_id $4)\"\n"
	);
	if(err == 1)
		ret = 2;


	err = config_setup_write(etc_file("scripts/upload_start.sh"), 1,
		"#!/bin/sh\n"
		"\n"
		"# upload_start.sh - args: <path+file> <file> <ip>\n"
		"logger -t fags \"Uploading '$2' to $3.\"\n"
	);
	if(err == 1)
		ret = 2;

	err = config_setup_write(etc_file("scripts/upload_end.sh"), 1,
		"#!/bin/sh\n"
		"\n"
		"# upload_end.sh - args: <path+file> <file> <avg speed>\n"
		"logger -t fags \"Uploaded '$2'. Average speed was $3KB/s.\"\n"
	);
	if(err == 1)
		ret = 2;

	err = config_setup_write(etc_file("scripts/delete.sh"), 1,
		"#!/bin/sh\n"
		"\n"
		"# delete.sh - args: <path> <file>\n"
		"echo \"Server wants us to delete '$2'\"\n"
	);
	if(err == 1)
		ret = 2;

	err = config_setup_write(etc_file("scripts/playfile.sh"), 1,
		"#!/bin/sh\n"
		"\n"
		"# playfile.sh - args: <op (play, queue or unknown)> <path> <file>\n"
		"echo \"Server wants us to $1 '$2/$3'\"\n"
	);
	if(err == 1)
		ret = 2;

	return ret;
}


